package com.persagy.dmp.rwd.define.controller;

import cn.hutool.core.collection.CollUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.node.ArrayNode;
import com.fasterxml.jackson.databind.node.ObjectNode;
import com.persagy.dmp.basic.model.QueryCriteria;
import com.persagy.dmp.basic.utils.QueryCriteriaHelper;
import com.persagy.dmp.common.constant.CommonConstant;
import com.persagy.dmp.common.constant.ResponseCode;
import com.persagy.dmp.common.context.AppContext;
import com.persagy.dmp.common.exception.BusinessException;
import com.persagy.dmp.common.helper.SpringHelper;
import com.persagy.dmp.common.model.response.CommonResult;
import com.persagy.dmp.common.utils.JsonHelper;
import com.persagy.dmp.common.utils.ResultHelper;
import com.persagy.dmp.define.entity.ObjectInfoDefine;
import com.persagy.dmp.define.entity.ObjectTypeDefine;
import com.persagy.dmp.digital.entity.ObjectDigital;
import com.persagy.dmp.mybatis.utils.ConditionUtil;
import com.persagy.dmp.rwd.define.service.IObjectInfoService;
import com.persagy.dmp.rwd.define.service.IObjectTypeService;
import lombok.RequiredArgsConstructor;
import org.apache.commons.lang.StringUtils;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.util.ArrayList;
import java.util.List;
import java.util.Set;

/**
 * 对象类型 Controller
 * @author Charlie Yu
 * @date 2021-06-25
 */
@RestController
@RequestMapping("/rwd/def/funcid")
@RequiredArgsConstructor
public class ObjectInfoController {

    private final IObjectInfoService service;
    private final IObjectTypeService objectTypeService;

    @PostMapping("")
    public CommonResult<List<ObjectInfoDefine>> query(@RequestBody QueryCriteria criteria) {
        if(criteria == null) {
            throw new BusinessException(ResponseCode.A0400.getCode(), ResponseCode.A0400.getDesc());
        }
        // 2021年8月18日20:06:24 兼容旧的查询
        ObjectNode objCriteria = criteria.getCriteria();
        if (null!=objCriteria && objCriteria.size()>0){
            objCriteria.remove(CommonConstant.TYPE);
        }
        List<ObjectTypeDefine> objectTypeDefines = queryClassCodes(criteria);
        // 1.如果传的参数不包含classCode走公共逻辑
        if (CollUtil.isEmpty(objectTypeDefines)){
            QueryWrapper<ObjectInfoDefine> wrapper = new QueryWrapper<>();
            // 添加所属项目条件
            ConditionUtil.ensureProjectCriteria(wrapper);
            // 转换查询条件
            QueryCriteriaHelper.toWrapper(wrapper, criteria, ObjectInfoDefine.class);
            Page<ObjectInfoDefine> page = QueryCriteriaHelper.toPage(criteria);
            // 没有排序条件时，按默认条件排序
            if(CollUtil.isEmpty(page.getOrders())) {
                wrapper.orderByAsc("show_order");
            }
            if(criteria.isOnlyCount()) {
                page = page.setTotal(service.queryCount(wrapper));
                return ResultHelper.multi(page.getRecords(), page.getTotal());
            }
            /// 20210819 临时处理，优化执行计划，解决Mysql:Out of sort memory问题
            // wrapper.select("/*+ SET_VAR(sort_buffer_size = 2M) */*");
            page = service.queryByCondition(QueryCriteriaHelper.toPage(criteria), wrapper);
            return ResultHelper.multi(page.getRecords(), page.getTotal());
        }
        // 2.包含classCode则不进行分页并且获取数据集合
        // 2.1 如果为基础类型、且为平台级，优先级100
        // 2.2 如果为基础类型、且为集团级，优先级200
        // 2.3 如果为基础类型、且为项目级，优先级300
        // 2.4 如果为非基础类型、且为平台级，优先级400
        // 2.5 如果为非基础类型、且为集团级，优先级500
        // 2.6 如果为非基础类型、且为项目级，优先级600
        criteria.getCriteria().remove(ObjectDigital.CLASS_CODE_HUM);
        List<ObjectInfoDefine> result = new ArrayList<>();
        for (ObjectTypeDefine objectTypeDefine : objectTypeDefines) {
            // 获取classCode集合
            Set<String> classCodes = CollUtil.newHashSet(objectTypeDefine.getCode());
            if (StrUtil.isNotBlank(objectTypeDefine.getParentCode())){
                classCodes.add(objectTypeDefine.getParentCode());
            }
            if (CollUtil.isEmpty(classCodes)){
                continue;
            }
            QueryWrapper<ObjectInfoDefine> wrapper = new QueryWrapper<>();
            // 添加所属项目条件
            ConditionUtil.ensureProjectCriteria(wrapper);
            // 转换查询条件
            QueryCriteriaHelper.toWrapper(wrapper, criteria, ObjectInfoDefine.class);
            wrapper.orderByAsc("show_order");
            wrapper.in(ObjectDigital.CLASS_CODE,classCodes);
            List<ObjectInfoDefine> objectInfoDefines = service.queryByClassCodes(wrapper,classCodes);
            if (CollUtil.isEmpty(objectInfoDefines)){
                continue;
            }
            objectInfoDefines.forEach(objectInfoDefine -> objectInfoDefine.setClassCode(objectTypeDefine.getCode()));
            result.addAll(objectInfoDefines);
        }
        return ResultHelper.multi(result, result.size());
    }

    @PostMapping("/create")
    public CommonResult<ObjectInfoDefine> create(@Valid @RequestBody ObjectInfoDefine vo){
        List<ObjectInfoDefine> voList = CollUtil.newArrayList(vo);
        //创建默认条件
        createDefaultValue(voList);
        //新增信息点
        service.insert(voList);
        return ResultHelper.single(vo);
    }

    @PostMapping("/createBatch")
    public CommonResult<List<ObjectInfoDefine>> createBatch(@Valid @RequestBody List<ObjectInfoDefine> voList){
        //创建默认条件
        createDefaultValue(voList);
        //批量新增信息点
        voList = service.insert(voList);
        return ResultHelper.multi(voList);
    }

    @PostMapping("/update")
    public CommonResult<ObjectInfoDefine> update(@Valid @RequestBody ObjectInfoDefine vo){
        List<ObjectInfoDefine> voList = CollUtil.newArrayList(vo);
        //创建默认条件
        createDefaultValue(voList);
        //修改信息点
        service.update(voList);
        return ResultHelper.single(vo);
    }

    @PostMapping("/updateBatch")
    public CommonResult<List<ObjectInfoDefine>> updateBatch(@Valid @RequestBody List<ObjectInfoDefine> voList){
        //创建默认条件
        createDefaultValue(voList);
        //修改信息点
        voList = service.update(voList);
        return ResultHelper.multi(voList);
    }

    @PostMapping("/delete")
    public CommonResult delete(@RequestBody String id){
        if(StringUtils.isEmpty(id)){
            throw new BusinessException(ResponseCode.A0400.getCode(), "id不能为空");
        }
        service.delete(CollUtil.newArrayList(id));
        return ResultHelper.success();
    }

    @PostMapping("/deleteBatch")
    public CommonResult deleteBatch(@RequestBody List<String> ids){
        if(ids == null || ids.size()<=0){
            throw new BusinessException(ResponseCode.A0400.getCode(), "id不能为空");
        }
        service.delete(ids);
        return ResultHelper.success();
    }

    /***
     * Description: 处理对象类型,如果父级id存在则加上父级id
     * @param criteria : 查询参数
     * @return : void
     * @author : lijie
     * @date :2021/9/9 18:25
     * Update By lijie 2021/9/9 18:25
     */
    private List<ObjectTypeDefine> queryClassCodes(QueryCriteria criteria) {
        ObjectNode objectNode = criteria.getCriteria();
        if (!objectNode.has(ObjectDigital.CLASS_CODE_HUM)){
            return null;
        }
        JsonNode jsonNode = objectNode.get(ObjectDigital.CLASS_CODE_HUM);
        QueryCriteria typeCriteria = new QueryCriteria();
        typeCriteria.setCriteria(SpringHelper.getBean(ObjectMapper.class).createObjectNode());
        typeCriteria.getCriteria().set("code",jsonNode);
        typeCriteria.getCriteria().put("valid",true);
        QueryWrapper<ObjectTypeDefine> wrapper = new QueryWrapper<>();
        // 添加所属项目条件
        ConditionUtil.ensureProjectCriteria(wrapper);
        // 转换查询条件
        QueryCriteriaHelper.toWrapper(wrapper, typeCriteria, ObjectTypeDefine.class);
        List<ObjectTypeDefine> objectTypeDefines = objectTypeService.queryByCondition(wrapper);
        if (CollUtil.isEmpty(objectTypeDefines)){
            return null;
        }
        return objectTypeDefines;
    }

    /**
     * 创建默认条件
     * @param voList
     */
    private void createDefaultValue(List<ObjectInfoDefine> voList){
        String groupCode = AppContext.getContext().getGroupCode();
        if(StringUtils.isEmpty(groupCode)){
            //默认集团
            groupCode = CommonConstant.DEFAULT_ID;
        }
        String projectId = AppContext.getContext().getProjectId();
        if(StringUtils.isEmpty(projectId)){
            //默认项目
            projectId = CommonConstant.DEFAULT_ID;
        }
        for (ObjectInfoDefine vo : voList) {
            vo.setGroupCode(groupCode);
            vo.setProjectId(projectId);
        }
    }
}
